@@ -7,6 +7,7 @@
 #include "int_flags.h"
 #include "setup.h"
 #include "liburing/io_uring.h"
+#include <stdio.h>
 
 #define KERN_MAX_ENTRIES	32768
 #define KERN_MAX_CQ_ENTRIES	(2 * KERN_MAX_ENTRIES)
@@ -94,18 +95,25 @@ void io_uring_setup_ring_pointers(struct io_uring_params *p,
 	cq->ring_entries = *cq->kring_entries;
 }
 
+static size_t params_sqes_size(const struct io_uring_params *p, unsigned sqes)
+{
+	sqes <<= io_uring_sqe_shift_from_flags(p->flags);
+	return sqes * sizeof(struct io_uring_sqe);
+}
+
+static size_t params_cq_size(const struct io_uring_params *p, unsigned cqes)
+{
+	cqes <<= io_uring_cqe_shift_from_flags(p->flags);
+	return cqes * sizeof(struct io_uring_cqe);
+}
+
 int io_uring_mmap(int fd, struct io_uring_params *p, struct io_uring_sq *sq,
 		  struct io_uring_cq *cq)
 {
-	size_t size;
 	int ret;
 
-	size = sizeof(struct io_uring_cqe);
-	if (p->flags & IORING_SETUP_CQE32)
-		size += sizeof(struct io_uring_cqe);
-
 	sq->ring_sz = p->sq_off.array + p->sq_entries * sizeof(unsigned);
-	cq->ring_sz = p->cq_off.cqes + p->cq_entries * size;
+	cq->ring_sz = p->cq_off.cqes + params_cq_size(p, p->cq_entries);
 
 	if (p->features & IORING_FEAT_SINGLE_MMAP) {
 		if (cq->ring_sz > sq->ring_sz)
@@ -131,11 +139,9 @@ int io_uring_mmap(int fd, struct io_uring_params *p, struct io_uring_sq *sq,
 		}
 	}
 
-	size = sizeof(struct io_uring_sqe);
-	if (p->flags & IORING_SETUP_SQE128)
-		size += 64;
-	sq->sqes = __sys_mmap(0, size * p->sq_entries, PROT_READ | PROT_WRITE,
-			      MAP_SHARED | MAP_POPULATE, fd, IORING_OFF_SQES);
+	sq->sqes = __sys_mmap(0, params_sqes_size(p, p->sq_entries),
+			      PROT_READ | PROT_WRITE, MAP_SHARED | MAP_POPULATE,
+			      fd, IORING_OFF_SQES);
 	if (IS_ERR(sq->sqes)) {
 		ret = PTR_ERR(sq->sqes);
 err:
@@ -160,6 +166,12 @@ __cold int io_uring_queue_mmap(int fd, struct io_uring_params *p,
 	return io_uring_mmap(fd, p, &ring->sq, &ring->cq);
 }
 
+static size_t io_uring_sqes_size(const struct io_uring *ring)
+{
+	return (ring->sq.ring_entries << io_uring_sqe_shift(ring)) *
+	       sizeof(struct io_uring_sqe);
+}
+
 /*
  * Ensure that the mmap'ed rings aren't available to a child after a fork(2).
  * This uses madvise(..., MADV_DONTFORK) on the mmap'ed ranges.
@@ -172,10 +184,7 @@ __cold int io_uring_ring_dontfork(struct io_uring *ring)
 	if (!ring->sq.ring_ptr || !ring->sq.sqes || !ring->cq.ring_ptr)
 		return -EINVAL;
 
-	len = sizeof(struct io_uring_sqe);
-	if (ring->flags & IORING_SETUP_SQE128)
-		len += 64;
-	len *= ring->sq.ring_entries;
+	len = io_uring_sqes_size(ring);
 	ret = __sys_madvise(ring->sq.sqes, len, MADV_DONTFORK);
 	if (ret < 0)
 		return ret;
@@ -209,7 +218,7 @@ static int io_uring_alloc_huge(unsigned entries, struct io_uring_params *p,
 {
 	unsigned long page_size = get_page_size();
 	unsigned sq_entries, cq_entries;
-	size_t ring_mem, sqes_mem, cqes_mem, sqe_size;
+	size_t ring_mem, sqes_mem;
 	unsigned long mem_used = 0;
 	void *ptr;
 	int ret;
@@ -220,18 +229,12 @@ static int io_uring_alloc_huge(unsigned entries, struct io_uring_params *p,
 
 	ring_mem = KRING_SIZE;
 
-	sqe_size = sizeof(struct io_uring_sqe);
-	if (p->flags & IORING_SETUP_SQE128)
-		sqe_size <<= 1;
-	sqes_mem = sq_entries * sqe_size;
+	sqes_mem = params_sqes_size(p, sq_entries);
 	if (!(p->flags & IORING_SETUP_NO_SQARRAY))
 		sqes_mem += sq_entries * sizeof(unsigned);
 	sqes_mem = (sqes_mem + page_size - 1) & ~(page_size - 1);
 
-	cqes_mem = cq_entries * sizeof(struct io_uring_cqe);
-	if (p->flags & IORING_SETUP_CQE32)
-		cqes_mem *= 2;
-	ring_mem += sqes_mem + cqes_mem;
+	ring_mem += sqes_mem + params_cq_size(p, cq_entries);
 	mem_used = ring_mem;
 	mem_used = (mem_used + page_size - 1) & ~(page_size - 1);
 
@@ -362,6 +365,13 @@ int __io_uring_queue_init_params(unsigned entries, struct io_uring *ring,
 	} else {
 		ring->ring_fd = fd;
 	}
+	/*
+	 * IOPOLL always needs to enter, except if SQPOLL is set as well.
+	 * Use an internal flag to check for this.
+	 */
+	if ((ring->flags & (IORING_SETUP_IOPOLL|IORING_SETUP_SQPOLL)) ==
+			IORING_SETUP_IOPOLL)
+		ring->int_flags |= INT_FLAG_CQ_ENTER;
 
 	return ret;
 }
@@ -433,19 +443,10 @@ __cold void io_uring_queue_exit(struct io_uring *ring)
 {
 	struct io_uring_sq *sq = &ring->sq;
 	struct io_uring_cq *cq = &ring->cq;
-	size_t sqe_size = sizeof(struct io_uring_sqe);
 
-	if (ring->flags & IORING_SETUP_SQE128)
-		sqe_size <<= 1;
-
-	if (!sq->ring_sz && !(ring->int_flags & INT_FLAG_APP_MEM)) {
-		__sys_munmap(sq->sqes, sqe_size * sq->ring_entries);
+	if (!(ring->int_flags & INT_FLAG_APP_MEM)) {
+		__sys_munmap(sq->sqes, io_uring_sqes_size(ring));
 		io_uring_unmap_rings(sq, cq);
-	} else {
-		if (!(ring->int_flags & INT_FLAG_APP_MEM)) {
-			__sys_munmap(sq->sqes, *sq->kring_entries * sqe_size);
-			io_uring_unmap_rings(sq, cq);
-		}
 	}
 
 	/*
@@ -498,34 +499,57 @@ __cold void io_uring_free_probe(struct io_uring_probe *probe)
 	free(probe);
 }
 
-static size_t npages(size_t size, long page_size)
-{
-	size--;
-	size /= page_size;
-	return __fls((int) size);
-}
-
 static size_t rings_size(struct io_uring_params *p, unsigned entries,
 			 unsigned cq_entries, long page_size)
 {
 	size_t pages, sq_size, cq_size;
 
-	cq_size = sizeof(struct io_uring_cqe);
-	if (p->flags & IORING_SETUP_CQE32)
-		cq_size += sizeof(struct io_uring_cqe);
-	cq_size *= cq_entries;
+	/*
+	 * CQ ring size is number of pages that we need for the
+	 * struct io_uring_cqe entries, which may be 16b (default) or
+	 * 32b if the ring is setup with IORING_SETUP_CQE32. We also need
+	 * room for the head/tail parts.
+	 */
+	cq_size = params_cq_size(p, cq_entries);
 	cq_size += KRING_SIZE;
-	cq_size = (cq_size + 63) & ~63UL;
-	pages = (size_t) 1 << npages(cq_size, page_size);
+	cq_size = (cq_size + page_size - 1) & ~(page_size - 1);
+	pages = (size_t) cq_size / page_size;
 
-	sq_size = sizeof(struct io_uring_sqe);
-	if (p->flags & IORING_SETUP_SQE128)
-		sq_size += 64;
-	sq_size *= entries;
-	pages += (size_t) 1 << npages(sq_size, page_size);
+	sq_size = params_sqes_size(p, entries);
+	sq_size = (sq_size + page_size - 1) & ~(page_size - 1);
+	pages += sq_size / page_size;
 	return pages * page_size;
 }
 
+ssize_t io_uring_memory_size_params(unsigned entries, struct io_uring_params *p)
+{
+	unsigned sq, cq;
+	long page_size;
+	ssize_t ret;
+
+	if (!entries)
+		return -EINVAL;
+	if (entries > KERN_MAX_ENTRIES) {
+		if (!(p->flags & IORING_SETUP_CLAMP))
+			return -EINVAL;
+		entries = KERN_MAX_ENTRIES;
+	}
+
+	ret = get_sq_cq_entries(entries, p, &sq, &cq);
+	if (ret)
+		return ret;
+
+	page_size = get_page_size();
+	return rings_size(p, sq, cq, page_size);
+}
+
+ssize_t io_uring_memory_size(unsigned entries, unsigned ring_flags)
+{
+	struct io_uring_params p = { .flags = ring_flags, };
+
+	return io_uring_memory_size_params(entries, &p);
+}
+
 /*
  * Return the required ulimit -l memlock memory required for a given ring
  * setup, in bytes. May return -errno on error. On newer (5.12+) kernels,
@@ -539,10 +563,7 @@ __cold ssize_t io_uring_mlock_size_params(unsigned entries,
 {
 	struct io_uring_params lp;
 	struct io_uring ring;
-	unsigned cq_entries, sq;
-	long page_size;
 	ssize_t ret;
-	int cret;
 
 	memset(&lp, 0, sizeof(lp));
 
@@ -564,20 +585,7 @@ __cold ssize_t io_uring_mlock_size_params(unsigned entries,
 	if (lp.features & IORING_FEAT_NATIVE_WORKERS)
 		return 0;
 
-	if (!entries)
-		return -EINVAL;
-	if (entries > KERN_MAX_ENTRIES) {
-		if (!(p->flags & IORING_SETUP_CLAMP))
-			return -EINVAL;
-		entries = KERN_MAX_ENTRIES;
-	}
-
-	cret = get_sq_cq_entries(entries, p, &sq, &cq_entries);
-	if (cret)
-		return cret;
-
-	page_size = get_page_size();
-	return rings_size(p, sq, cq_entries, page_size);
+	return io_uring_memory_size_params(entries, p);
 }
 
 /*
