@@ -15,15 +15,17 @@
 #include "liburing.h"
 #include "helpers.h"
 
-static char str[] = "This is a test of send and recv over io_uring!";
+#define MAX_MSG	4096
 
-#define MAX_MSG	128
+static unsigned long str[MAX_MSG / sizeof(unsigned long)];
 
 #define PORT	10202
 #define HOST	"127.0.0.1"
 
+static int no_send_vec;
+
 static int recv_prep(struct io_uring *ring, struct iovec *iov, int *sock,
-		     int registerfiles)
+		     int registerfiles, int async, int provide)
 {
 	struct sockaddr_in saddr;
 	struct io_uring_sqe *sqe;
@@ -64,6 +66,10 @@ static int recv_prep(struct io_uring *ring, struct iovec *iov, int *sock,
 	io_uring_prep_recv(sqe, use_fd, iov->iov_base, iov->iov_len, 0);
 	if (registerfiles)
 		sqe->flags |= IOSQE_FIXED_FILE;
+	if (async)
+		sqe->flags |= IOSQE_ASYNC;
+	if (provide)
+		sqe->flags |= IOSQE_BUFFER_SELECT;
 	sqe->user_data = 2;
 
 	ret = io_uring_submit(ring);
@@ -79,38 +85,51 @@ err:
 	return 1;
 }
 
-static int do_recv(struct io_uring *ring, struct iovec *iov)
+static int do_recv(struct io_uring *ring, struct iovec *iov, int enobufs)
 {
 	struct io_uring_cqe *cqe;
-	int ret;
+	unsigned long *ptr;
+	int i, ret;
 
 	ret = io_uring_wait_cqe(ring, &cqe);
 	if (ret) {
 		fprintf(stdout, "wait_cqe: %d\n", ret);
-		goto err;
+		return 1;
 	}
 	if (cqe->res == -EINVAL) {
 		fprintf(stdout, "recv not supported, skipping\n");
-		return 0;
+		goto out;
+	}
+	if (cqe->res == -ENOBUFS && enobufs) {
+		if (cqe->flags & IORING_CQE_F_SOCK_NONEMPTY) {
+			fprintf(stdout, "NONEMPTY set on -ENOBUFS\n");
+			goto err;
+		}
+		goto out;
 	}
 	if (cqe->res < 0) {
 		fprintf(stderr, "failed cqe: %d\n", cqe->res);
 		goto err;
 	}
 
-	if (cqe->res -1 != strlen(str)) {
-		fprintf(stderr, "got wrong length: %d/%d\n", cqe->res,
-							(int) strlen(str) + 1);
+	if (cqe->res != MAX_MSG) {
+		fprintf(stderr, "got wrong length: %d/%d\n", cqe->res, MAX_MSG);
 		goto err;
 	}
 
-	if (strcmp(str, iov->iov_base)) {
-		fprintf(stderr, "string mismatch\n");
+	ptr = iov->iov_base;
+	for (i = 0; i < MAX_MSG / sizeof(unsigned long); i++) {
+		if (ptr[i] == str[i])
+			continue;
+		fprintf(stderr, "data mismatch at %d: %lu\n", i, ptr[i]);
 		goto err;
 	}
 
+out:
+	io_uring_cqe_seen(ring, cqe);
 	return 0;
 err:
+	io_uring_cqe_seen(ring, cqe);
 	return 1;
 }
 
@@ -118,6 +137,8 @@ struct recv_data {
 	pthread_mutex_t mutex;
 	int use_sqthread;
 	int registerfiles;
+	int async;
+	int provide;
 };
 
 static void *recv_fn(void *data)
@@ -152,13 +173,14 @@ static void *recv_fn(void *data)
 		}
 	}
 
-	ret = recv_prep(&ring, &iov, &sock, rd->registerfiles);
+	ret = recv_prep(&ring, &iov, &sock, rd->registerfiles, rd->async,
+				rd->provide);
 	if (ret) {
 		fprintf(stderr, "recv_prep failed: %d\n", ret);
 		goto err;
 	}
 	pthread_mutex_unlock(&rd->mutex);
-	ret = do_recv(&ring, &iov);
+	ret = do_recv(&ring, &iov, rd->provide);
 
 	close(sock);
 	io_uring_queue_exit(&ring);
@@ -166,13 +188,10 @@ err:
 	return (void *)(intptr_t)ret;
 }
 
-static int do_send(void)
+static int do_send(int async, int vec, int big_vec)
 {
 	struct sockaddr_in saddr;
-	struct iovec iov = {
-		.iov_base = str,
-		.iov_len = sizeof(str),
-	};
+	struct iovec vecs[32];
 	struct io_uring ring;
 	struct io_uring_cqe *cqe;
 	struct io_uring_sqe *sqe;
@@ -201,8 +220,37 @@ static int do_send(void)
 		goto err;
 	}
 
+retry:
 	sqe = io_uring_get_sqe(&ring);
-	io_uring_prep_send(sqe, sockfd, iov.iov_base, iov.iov_len, 0);
+	if (vec) {
+		size_t total = MAX_MSG;
+		unsigned long *ptr = str;
+		int i, nvecs;
+
+		if (!big_vec) {
+			vecs[0].iov_base = str;
+			vecs[0].iov_len = MAX_MSG / 2;
+			vecs[1].iov_base = &str[256];
+			vecs[1].iov_len = MAX_MSG / 2;
+			nvecs = 2;
+		} else {
+			total /= 32;
+
+			for (i = 0; i < 32; i++) {
+				vecs[i].iov_base = ptr;
+				vecs[i].iov_len = total;
+				ptr += total / sizeof(unsigned long);
+			}
+			nvecs = 32;
+		}
+
+		io_uring_prep_send(sqe, sockfd, vecs, nvecs, 0);
+		sqe->ioprio = (1U << 5);
+	} else {
+		io_uring_prep_send(sqe, sockfd, str, sizeof(str), 0);
+	}
+	if (async)
+		sqe->flags |= IOSQE_ASYNC;
 	sqe->user_data = 1;
 
 	ret = io_uring_submit(&ring);
@@ -213,10 +261,16 @@ static int do_send(void)
 
 	ret = io_uring_wait_cqe(&ring, &cqe);
 	if (cqe->res == -EINVAL) {
+		if (vec) {
+			vec = 0;
+			no_send_vec = 1;
+			io_uring_cqe_seen(&ring, cqe);
+			goto retry;
+		}
 		fprintf(stdout, "send not supported, skipping\n");
 		goto err;
 	}
-	if (cqe->res != iov.iov_len) {
+	if (cqe->res != sizeof(str)) {
 		fprintf(stderr, "failed cqe: %d\n", cqe->res);
 		goto err;
 	}
@@ -232,7 +286,8 @@ err2:
 	return 1;
 }
 
-static int test(int use_sqthread, int regfiles)
+static int test(int use_sqthread, int regfiles, int async, int provide, int vec,
+		int big_vec)
 {
 	pthread_mutexattr_t attr;
 	pthread_t recv_thread;
@@ -240,12 +295,17 @@ static int test(int use_sqthread, int regfiles)
 	int ret;
 	void *retval;
 
+	if (vec && no_send_vec)
+		return T_EXIT_SKIP;
+
 	pthread_mutexattr_init(&attr);
 	pthread_mutexattr_setpshared(&attr, 1);
 	pthread_mutex_init(&rd.mutex, &attr);
 	pthread_mutex_lock(&rd.mutex);
 	rd.use_sqthread = use_sqthread;
 	rd.registerfiles = regfiles;
+	rd.async = async;
+	rd.provide = provide;
 
 	ret = pthread_create(&recv_thread, NULL, recv_fn, &rd);
 	if (ret) {
@@ -255,7 +315,7 @@ static int test(int use_sqthread, int regfiles)
 	}
 
 	pthread_mutex_lock(&rd.mutex);
-	do_send();
+	do_send(async, vec, big_vec);
 	pthread_join(recv_thread, &retval);
 	return (intptr_t)retval;
 }
@@ -306,10 +366,13 @@ static int test_invalid(void)
 
 int main(int argc, char *argv[])
 {
-	int ret;
+	int i, ret;
 
 	if (argc > 1)
-		return 0;
+		return T_EXIT_SKIP;
+
+	for (i = 0; i < MAX_MSG / sizeof(unsigned long); i++)
+		str[i] = i + 1;
 
 	ret = test_invalid();
 	if (ret) {
@@ -317,23 +380,103 @@ int main(int argc, char *argv[])
 		return ret;
 	}
 
-	ret = test(0, 0);
+	ret = test(0, 0, 1, 1, 0, 0);
 	if (ret) {
-		fprintf(stderr, "test sqthread=0 failed\n");
+		fprintf(stderr, "test sqthread=0 1 1 failed\n");
 		return ret;
 	}
 
-	ret = test(1, 1);
+	ret = test(1, 1, 1, 1, 0, 0);
 	if (ret) {
-		fprintf(stderr, "test sqthread=1 reg=1 failed\n");
+		fprintf(stderr, "test sqthread=1 reg=1 1 1 failed\n");
 		return ret;
 	}
 
-	ret = test(1, 0);
+	ret = test(1, 0, 1, 1, 0, 0);
 	if (ret) {
-		fprintf(stderr, "test sqthread=1 reg=0 failed\n");
+		fprintf(stderr, "test sqthread=1 reg=0 1 1 failed\n");
 		return ret;
 	}
 
-	return 0;
+	ret = test(0, 0, 0, 1, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=0 0 1 failed\n");
+		return ret;
+	}
+
+	ret = test(1, 1, 0, 1, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=1 reg=1 0 1 failed\n");
+		return ret;
+	}
+
+	ret = test(1, 0, 0, 1, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=1 reg=0 0 1 failed\n");
+		return ret;
+	}
+
+	ret = test(0, 0, 1, 0, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=0 0 1 failed\n");
+		return ret;
+	}
+
+	ret = test(1, 1, 1, 0, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=1 reg=1 1 0 failed\n");
+		return ret;
+	}
+
+	ret = test(1, 0, 1, 0, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=1 reg=0 1 0 failed\n");
+		return ret;
+	}
+
+	ret = test(0, 0, 0, 0, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=0 0 0 failed\n");
+		return ret;
+	}
+
+	ret = test(1, 1, 0, 0, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=1 reg=1 0 0 failed\n");
+		return ret;
+	}
+
+	ret = test(1, 0, 0, 0, 0, 0);
+	if (ret) {
+		fprintf(stderr, "test sqthread=1 reg=0 0 0 failed\n");
+		return ret;
+	}
+
+	ret = test(0, 0, 0, 0, 1, 0);
+	if (ret) {
+		fprintf(stderr, "test small vec sync failed\n");
+		return ret;
+	}
+	if (no_send_vec)
+		return T_EXIT_PASS;
+
+	ret = test(0, 0, 1, 0, 1, 0);
+	if (ret) {
+		fprintf(stderr, "test small vec async failed\n");
+		return ret;
+	}
+
+	ret = test(0, 0, 0, 0, 1, 1);
+	if (ret) {
+		fprintf(stderr, "test big vec sync failed\n");
+		return ret;
+	}
+
+	ret = test(0, 0, 1, 0, 1, 1);
+	if (ret) {
+		fprintf(stderr, "test big vec async failed\n");
+		return ret;
+	}
+
+	return T_EXIT_PASS;
 }
